"""v1.2.0-name-constraints

Revision ID: e382f30fad09
Revises: 85c7256cc390
Create Date: 2024-06-02 15:46:28.362288

"""

from typing import Sequence, Union

import sqlalchemy as sa
import sqlmodel
from alembic import op


# revision identifiers, used by Alembic.
revision: str = "e382f30fad09"
down_revision: Union[str, None] = "85c7256cc390"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Recreate tables with named constraints.

    Need to recreate all tables with FKs to name those, as were
    created without a name previously, and there is no way to
    reference those in Alembic. Now with automatic naming schema, this
    should be fixed.

    References: https://alembic.sqlalchemy.org/en/latest/naming.html
    """
    # TODO rename all tables
    # TODO create new tables with added constraints with copy_from
    # TODO drop old tables
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "benchmark",
        sa.Column(
            "benchmark_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier of a specific Benchmark.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short description.",
        ),
        sa.Column(
            "framework",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="The name of the benchmark framework/software/tool used.",
        ),
        sa.Column(
            "config_fields",
            sa.JSON(),
            nullable=False,
            comment='A dictionary of descriptions on the framework-specific config options, e.g. {"bandwidth": "Memory amount to use for compression in MB."}.',
        ),
        sa.Column(
            "measurement",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The name of measurement recoreded in the benchmark.",
        ),
        sa.Column(
            "unit",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional unit of measurement for the benchmark score.",
        ),
        sa.Column(
            "higher_is_better",
            sa.Boolean(),
            nullable=False,
            comment="If higher benchmark score means better performance, or vica versa.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.PrimaryKeyConstraint("benchmark_id", name=op.f("pk_benchmark")),
        comment="Benchmark scenario definitions.",
    )
    op.create_table(
        "compliance_framework",
        sa.Column(
            "compliance_framework_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "abbreviation",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short abbreviation of the Framework name.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Description of the framework in a few paragrahs, outlining key features and characteristics for reference.",
        ),
        sa.Column(
            "logo",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Publicly accessible URL to the image of the Framework's logo.",
        ),
        sa.Column(
            "homepage",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Public homepage with more information on the Framework.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.PrimaryKeyConstraint(
            "compliance_framework_id", name=op.f("pk_compliance_framework")
        ),
        comment="List of Compliance Frameworks, such as HIPAA or SOC 2 Type 1.",
    )
    op.create_table(
        "compliance_framework_scd",
        sa.Column(
            "compliance_framework_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "abbreviation",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short abbreviation of the Framework name.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Description of the framework in a few paragrahs, outlining key features and characteristics for reference.",
        ),
        sa.Column(
            "logo",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Publicly accessible URL to the image of the Framework's logo.",
        ),
        sa.Column(
            "homepage",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Public homepage with more information on the Framework.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.PrimaryKeyConstraint(
            "compliance_framework_id",
            "observed_at",
            name=op.f("pk_compliance_framework_scd"),
        ),
        comment="SCD version of .tables.ComplianceFrameworkScd.",
    )
    op.create_table(
        "country",
        sa.Column(
            "country_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Country code by ISO 3166 alpha-2.",
        ),
        sa.Column(
            "continent",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Continent name.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.PrimaryKeyConstraint("country_id"),
        comment="Country and continent mapping.",
    )
    op.create_table(
        "country_scd",
        sa.Column(
            "country_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Country code by ISO 3166 alpha-2.",
        ),
        sa.Column(
            "continent",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Continent name.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.PrimaryKeyConstraint(
            "country_id", "observed_at", name=op.f("pk_country_scd")
        ),
        comment="SCD version of .tables.Country.",
    )
    op.create_table(
        "vendor",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "logo",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Publicly accessible URL to the image of the Vendor's logo.",
        ),
        sa.Column(
            "homepage",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Public homepage of the Vendor.",
        ),
        sa.Column(
            "country_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Country, where the Vendor's main headquarter is located.",
        ),
        sa.Column(
            "state",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional state/administrative area of the Vendor's location within the Country.",
        ),
        sa.Column(
            "city",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional city name of the Vendor's main location.",
        ),
        sa.Column(
            "address_line",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional address line of the Vendor's main location.",
        ),
        sa.Column(
            "zip_code",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional ZIP code of the Vendor's main location.",
        ),
        sa.Column(
            "founding_year",
            sa.Integer(),
            nullable=False,
            comment="4-digit year when the Vendor was founded.",
        ),
        sa.Column(
            "status_page",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Public status page of the Vendor.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["country_id"],
            ["country.country_id"],
            name=op.f("fk_vendor_country_id_country"),
        ),
        sa.PrimaryKeyConstraint("vendor_id", name=op.f("pk_vendor")),
        comment="Compute resource vendors, such as cloud and server providers.",
    )
    op.create_table(
        "vendor_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "logo",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Publicly accessible URL to the image of the Vendor's logo.",
        ),
        sa.Column(
            "homepage",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Public homepage of the Vendor.",
        ),
        sa.Column(
            "country_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Country, where the Vendor's main headquarter is located.",
        ),
        sa.Column(
            "state",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional state/administrative area of the Vendor's location within the Country.",
        ),
        sa.Column(
            "city",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional city name of the Vendor's main location.",
        ),
        sa.Column(
            "address_line",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional address line of the Vendor's main location.",
        ),
        sa.Column(
            "zip_code",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional ZIP code of the Vendor's main location.",
        ),
        sa.Column(
            "founding_year",
            sa.Integer(),
            nullable=False,
            comment="4-digit year when the Vendor was founded.",
        ),
        sa.Column(
            "status_page",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Public status page of the Vendor.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["country_id"],
            ["country.country_id"],
            name=op.f("fk_vendor_scd_country_id_country"),
        ),
        sa.PrimaryKeyConstraint("vendor_id", "observed_at", name=op.f("pk_vendor_scd")),
        comment="SCD version of .tables.VendorScd.",
    )
    op.create_table(
        "datacenter",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "api_reference",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="How this resource is referenced in the vendor API calls. This is usually either the id or name of the resource, depening on the vendor and actual API endpoint.",
        ),
        sa.Column(
            "display_name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly reference (usually the id or name) of the resource.",
        ),
        sa.Column(
            "aliases",
            sa.JSON(),
            nullable=False,
            comment="List of other commonly used names for the same Datacenter.",
        ),
        sa.Column(
            "country_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Country, where the Datacenter is located.",
        ),
        sa.Column(
            "state",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional state/administrative area of the Datacenter's location within the Country.",
        ),
        sa.Column(
            "city",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional city name of the Datacenter's location.",
        ),
        sa.Column(
            "address_line",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional address line of the Datacenter's location.",
        ),
        sa.Column(
            "zip_code",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional ZIP code of the Datacenter's location.",
        ),
        sa.Column(
            "lon",
            sa.Float(),
            nullable=True,
            comment="Longitude coordinate of the Datacenter's known or approximate location.",
        ),
        sa.Column(
            "lat",
            sa.Float(),
            nullable=True,
            comment="Latitude coordinate of the Datacenter's known or approximate location.",
        ),
        sa.Column(
            "founding_year",
            sa.Integer(),
            nullable=True,
            comment="4-digit year when the Datacenter was founded.",
        ),
        sa.Column(
            "green_energy",
            sa.Boolean(),
            nullable=True,
            comment="If the Datacenter is 100% powered by renewable energy.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["country_id"],
            ["country.country_id"],
            name=op.f("fk_datacenter_country_id_country"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_datacenter_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", name=op.f("pk_datacenter")
        ),
        comment="Datacenters/regions of Vendors.",
    )
    op.create_table(
        "datacenter_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "api_reference",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="How this resource is referenced in the vendor API calls. This is usually either the id or name of the resource, depening on the vendor and actual API endpoint.",
        ),
        sa.Column(
            "display_name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly reference (usually the id or name) of the resource.",
        ),
        sa.Column(
            "aliases",
            sa.JSON(),
            nullable=False,
            comment="List of other commonly used names for the same Datacenter.",
        ),
        sa.Column(
            "country_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Country, where the Datacenter is located.",
        ),
        sa.Column(
            "state",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional state/administrative area of the Datacenter's location within the Country.",
        ),
        sa.Column(
            "city",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional city name of the Datacenter's location.",
        ),
        sa.Column(
            "address_line",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional address line of the Datacenter's location.",
        ),
        sa.Column(
            "zip_code",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional ZIP code of the Datacenter's location.",
        ),
        sa.Column(
            "lon",
            sa.Float(),
            nullable=True,
            comment="Longitude coordinate of the Datacenter's known or approximate location.",
        ),
        sa.Column(
            "lat",
            sa.Float(),
            nullable=True,
            comment="Latitude coordinate of the Datacenter's known or approximate location.",
        ),
        sa.Column(
            "founding_year",
            sa.Integer(),
            nullable=True,
            comment="4-digit year when the Datacenter was founded.",
        ),
        sa.Column(
            "green_energy",
            sa.Boolean(),
            nullable=True,
            comment="If the Datacenter is 100% powered by renewable energy.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["country_id"],
            ["country.country_id"],
            name=op.f("fk_datacenter_scd_country_id_country"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_datacenter_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", "observed_at", name=op.f("pk_datacenter_scd")
        ),
        comment="SCD version of .tables.DatacenterScd.",
    )
    op.create_table(
        "ipv4_price_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_ipv4_price_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", "observed_at", name=op.f("pk_ipv4_price_scd")
        ),
        comment="SCD version of .tables.Ipv4PriceScd.",
    )
    op.create_table(
        "server",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "server_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "api_reference",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="How this resource is referenced in the vendor API calls. This is usually either the id or name of the resource, depening on the vendor and actual API endpoint.",
        ),
        sa.Column(
            "display_name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly reference (usually the id or name) of the resource.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short description.",
        ),
        sa.Column(
            "family",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Server family, e.g. General-purpose machine (GCP), or M5g (AWS).",
        ),
        sa.Column(
            "vcpus",
            sa.Integer(),
            nullable=False,
            comment="Default number of virtual CPUs (vCPU) of the server.",
        ),
        sa.Column(
            "hypervisor",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Hypervisor of the virtual server, e.g. Xen, KVM, Nitro or Dedicated.",
        ),
        sa.Column(
            "cpu_allocation",
            sa.Enum("SHARED", "BURSTABLE", "DEDICATED", name="cpuallocation"),
            nullable=False,
            comment="Allocation of CPU(s) to the server, e.g. shared, burstable or dedicated.",
        ),
        sa.Column(
            "cpu_cores",
            sa.Integer(),
            nullable=True,
            comment="Default number of CPU cores of the server. Equals to vCPUs when HyperThreading is disabled.",
        ),
        sa.Column(
            "cpu_speed",
            sa.Float(),
            nullable=True,
            comment="Vendor-reported maximum CPU clock speed (GHz).",
        ),
        sa.Column(
            "cpu_architecture",
            sa.Enum(
                "ARM64",
                "ARM64_MAC",
                "I386",
                "X86_64",
                "X86_64_MAC",
                name="cpuarchitecture",
            ),
            nullable=False,
            comment="CPU architecture (arm64, arm64_mac, i386, or x86_64).",
        ),
        sa.Column(
            "cpu_manufacturer",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The manufacturer of the primary processor, e.g. Intel or AMD.",
        ),
        sa.Column(
            "cpu_family",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The product line/family of the primary processor, e.g. Xeon, Core i7, Ryzen 9.",
        ),
        sa.Column(
            "cpu_model",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The model number of the primary processor, e.g. 9750H.",
        ),
        sa.Column(
            "cpu_l1_cache", sa.Integer(), nullable=True, comment="L1 cache size (MiB)."
        ),
        sa.Column(
            "cpu_l2_cache", sa.Integer(), nullable=True, comment="L2 cache size (MiB)."
        ),
        sa.Column(
            "cpu_l3_cache", sa.Integer(), nullable=True, comment="L3 cache size (MiB)."
        ),
        sa.Column(
            "cpu_flags", sa.JSON(), nullable=False, comment="CPU features/flags."
        ),
        sa.Column(
            "cpus",
            sa.JSON(),
            nullable=False,
            comment="JSON array of known CPU details, e.g. the manufacturer, family, model; L1/L2/L3 cache size; microcode version; feature flags; bugs etc.",
        ),
        sa.Column(
            "memory_amount", sa.Integer(), nullable=False, comment="RAM amount (MiB)."
        ),
        sa.Column(
            "memory_generation",
            sa.Enum("DDR3", "DDR4", "DDR5", name="ddrgeneration"),
            nullable=True,
            comment="Generation of the DDR SDRAM, e.g. DDR4 or DDR5.",
        ),
        sa.Column(
            "memory_speed",
            sa.Integer(),
            nullable=True,
            comment="DDR SDRAM clock rate (Mhz).",
        ),
        sa.Column(
            "memory_ecc",
            sa.Boolean(),
            nullable=True,
            comment="If the DDR SDRAM uses error correction code to detect and correct n-bit data corruption.",
        ),
        sa.Column(
            "gpu_count",
            sa.Integer(),
            nullable=False,
            comment="Number of GPU accelerator(s).",
        ),
        sa.Column(
            "gpu_memory_min",
            sa.Integer(),
            nullable=True,
            comment="Memory (MiB) allocated to the lowest-end GPU accelerator.",
        ),
        sa.Column(
            "gpu_memory_total",
            sa.Integer(),
            nullable=True,
            comment="Overall memory (MiB) allocated to all the GPU accelerator(s).",
        ),
        sa.Column(
            "gpu_manufacturer",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The manufacturer of the primary GPU accelerator, e.g. Nvidia or AMD.",
        ),
        sa.Column(
            "gpu_family",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The product family of the primary GPU accelerator, e.g. Turing.",
        ),
        sa.Column(
            "gpu_model",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The model number of the primary GPU accelerator, e.g. Tesla T4.",
        ),
        sa.Column(
            "gpus",
            sa.JSON(),
            nullable=False,
            comment="JSON array of GPU accelerator details, including the manufacturer, name, and memory (MiB) of each GPU.",
        ),
        sa.Column(
            "storage_size",
            sa.Integer(),
            nullable=False,
            comment="Overall size (GB) of the disk(s).",
        ),
        sa.Column(
            "storage_type",
            sa.Enum("HDD", "SSD", "NVME_SSD", "NETWORK", name="storagetype"),
            nullable=True,
            comment="Primary disk type, e.g. HDD, SSD, NVMe SSD, or network).",
        ),
        sa.Column(
            "storages",
            sa.JSON(),
            nullable=False,
            comment="JSON array of disks attached to the server, including the size (MiB) and type of each disk.",
        ),
        sa.Column(
            "network_speed",
            sa.Float(),
            nullable=True,
            comment="The baseline network performance (Gbps) of the network card.",
        ),
        sa.Column(
            "inbound_traffic",
            sa.Float(),
            nullable=False,
            comment="Amount of complimentary inbound traffic (GB) per month.",
        ),
        sa.Column(
            "outbound_traffic",
            sa.Float(),
            nullable=False,
            comment="Amount of complimentary outbound traffic (GB) per month.",
        ),
        sa.Column(
            "ipv4",
            sa.Integer(),
            nullable=False,
            comment="Number of complimentary IPv4 address(es).",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"], ["vendor.vendor_id"], name=op.f("fk_server_vendor_id_vendor")
        ),
        sa.PrimaryKeyConstraint("vendor_id", "server_id", name=op.f("pk_server")),
        comment="Server types.",
    )
    op.create_table(
        "server_price_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "zone_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Zone.",
        ),
        sa.Column(
            "server_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Server.",
        ),
        sa.Column(
            "operating_system",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Operating System.",
        ),
        sa.Column(
            "allocation",
            sa.Enum("ONDEMAND", "RESERVED", "SPOT", name="allocation"),
            nullable=False,
            comment="Allocation method, e.g. on-demand or spot.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_server_price_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "datacenter_id",
            "zone_id",
            "server_id",
            "allocation",
            "observed_at",
            name=op.f("pk_server_price_scd"),
        ),
        comment="SCD version of .tables.ServerPriceScd.",
    )
    op.create_table(
        "server_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "server_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "api_reference",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="How this resource is referenced in the vendor API calls. This is usually either the id or name of the resource, depening on the vendor and actual API endpoint.",
        ),
        sa.Column(
            "display_name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly reference (usually the id or name) of the resource.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short description.",
        ),
        sa.Column(
            "family",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Server family, e.g. General-purpose machine (GCP), or M5g (AWS).",
        ),
        sa.Column(
            "vcpus",
            sa.Integer(),
            nullable=False,
            comment="Default number of virtual CPUs (vCPU) of the server.",
        ),
        sa.Column(
            "hypervisor",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Hypervisor of the virtual server, e.g. Xen, KVM, Nitro or Dedicated.",
        ),
        sa.Column(
            "cpu_allocation",
            sa.Enum("SHARED", "BURSTABLE", "DEDICATED", name="cpuallocation"),
            nullable=False,
            comment="Allocation of CPU(s) to the server, e.g. shared, burstable or dedicated.",
        ),
        sa.Column(
            "cpu_cores",
            sa.Integer(),
            nullable=True,
            comment="Default number of CPU cores of the server. Equals to vCPUs when HyperThreading is disabled.",
        ),
        sa.Column(
            "cpu_speed",
            sa.Float(),
            nullable=True,
            comment="Vendor-reported maximum CPU clock speed (GHz).",
        ),
        sa.Column(
            "cpu_architecture",
            sa.Enum(
                "ARM64",
                "ARM64_MAC",
                "I386",
                "X86_64",
                "X86_64_MAC",
                name="cpuarchitecture",
            ),
            nullable=False,
            comment="CPU architecture (arm64, arm64_mac, i386, or x86_64).",
        ),
        sa.Column(
            "cpu_manufacturer",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The manufacturer of the primary processor, e.g. Intel or AMD.",
        ),
        sa.Column(
            "cpu_family",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The product line/family of the primary processor, e.g. Xeon, Core i7, Ryzen 9.",
        ),
        sa.Column(
            "cpu_model",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The model number of the primary processor, e.g. 9750H.",
        ),
        sa.Column(
            "cpu_l1_cache", sa.Integer(), nullable=True, comment="L1 cache size (MiB)."
        ),
        sa.Column(
            "cpu_l2_cache", sa.Integer(), nullable=True, comment="L2 cache size (MiB)."
        ),
        sa.Column(
            "cpu_l3_cache", sa.Integer(), nullable=True, comment="L3 cache size (MiB)."
        ),
        sa.Column(
            "cpu_flags", sa.JSON(), nullable=False, comment="CPU features/flags."
        ),
        sa.Column(
            "cpus",
            sa.JSON(),
            nullable=False,
            comment="JSON array of known CPU details, e.g. the manufacturer, family, model; L1/L2/L3 cache size; microcode version; feature flags; bugs etc.",
        ),
        sa.Column(
            "memory_amount", sa.Integer(), nullable=False, comment="RAM amount (MiB)."
        ),
        sa.Column(
            "memory_generation",
            sa.Enum("DDR3", "DDR4", "DDR5", name="ddrgeneration"),
            nullable=True,
            comment="Generation of the DDR SDRAM, e.g. DDR4 or DDR5.",
        ),
        sa.Column(
            "memory_speed",
            sa.Integer(),
            nullable=True,
            comment="DDR SDRAM clock rate (Mhz).",
        ),
        sa.Column(
            "memory_ecc",
            sa.Boolean(),
            nullable=True,
            comment="If the DDR SDRAM uses error correction code to detect and correct n-bit data corruption.",
        ),
        sa.Column(
            "gpu_count",
            sa.Integer(),
            nullable=False,
            comment="Number of GPU accelerator(s).",
        ),
        sa.Column(
            "gpu_memory_min",
            sa.Integer(),
            nullable=True,
            comment="Memory (MiB) allocated to the lowest-end GPU accelerator.",
        ),
        sa.Column(
            "gpu_memory_total",
            sa.Integer(),
            nullable=True,
            comment="Overall memory (MiB) allocated to all the GPU accelerator(s).",
        ),
        sa.Column(
            "gpu_manufacturer",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The manufacturer of the primary GPU accelerator, e.g. Nvidia or AMD.",
        ),
        sa.Column(
            "gpu_family",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The product family of the primary GPU accelerator, e.g. Turing.",
        ),
        sa.Column(
            "gpu_model",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="The model number of the primary GPU accelerator, e.g. Tesla T4.",
        ),
        sa.Column(
            "gpus",
            sa.JSON(),
            nullable=False,
            comment="JSON array of GPU accelerator details, including the manufacturer, name, and memory (MiB) of each GPU.",
        ),
        sa.Column(
            "storage_size",
            sa.Integer(),
            nullable=False,
            comment="Overall size (GB) of the disk(s).",
        ),
        sa.Column(
            "storage_type",
            sa.Enum("HDD", "SSD", "NVME_SSD", "NETWORK", name="storagetype"),
            nullable=True,
            comment="Primary disk type, e.g. HDD, SSD, NVMe SSD, or network).",
        ),
        sa.Column(
            "storages",
            sa.JSON(),
            nullable=False,
            comment="JSON array of disks attached to the server, including the size (MiB) and type of each disk.",
        ),
        sa.Column(
            "network_speed",
            sa.Float(),
            nullable=True,
            comment="The baseline network performance (Gbps) of the network card.",
        ),
        sa.Column(
            "inbound_traffic",
            sa.Float(),
            nullable=False,
            comment="Amount of complimentary inbound traffic (GB) per month.",
        ),
        sa.Column(
            "outbound_traffic",
            sa.Float(),
            nullable=False,
            comment="Amount of complimentary outbound traffic (GB) per month.",
        ),
        sa.Column(
            "ipv4",
            sa.Integer(),
            nullable=False,
            comment="Number of complimentary IPv4 address(es).",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_server_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "server_id", "observed_at", name=op.f("pk_server_scd")
        ),
        comment="SCD version of .tables.ServerScd.",
    )
    op.create_table(
        "storage",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "storage_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short description.",
        ),
        sa.Column(
            "storage_type",
            sa.Enum("HDD", "SSD", "NVME_SSD", "NETWORK", name="storagetype"),
            nullable=False,
            comment="High-level category of the storage, e.g. HDD or SDD.",
        ),
        sa.Column(
            "max_iops",
            sa.Integer(),
            nullable=True,
            comment="Maximum Input/Output Operations Per Second.",
        ),
        sa.Column(
            "max_throughput",
            sa.Integer(),
            nullable=True,
            comment="Maximum Throughput (MiB/s).",
        ),
        sa.Column(
            "min_size",
            sa.Integer(),
            nullable=True,
            comment="Minimum required size (GiB).",
        ),
        sa.Column(
            "max_size",
            sa.Integer(),
            nullable=True,
            comment="Maximum possible size (GiB).",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_storage_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint("vendor_id", "storage_id", name=op.f("pk_storage")),
        comment="Flexible storage options that can be attached to a Server.",
    )
    op.create_table(
        "storage_price_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "storage_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Storage.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_storage_price_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "datacenter_id",
            "storage_id",
            "observed_at",
            name=op.f("pk_storage_price_scd"),
        ),
        comment="SCD version of .tables.StoragePriceScd.",
    )
    op.create_table(
        "storage_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "storage_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "description",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Short description.",
        ),
        sa.Column(
            "storage_type",
            sa.Enum("HDD", "SSD", "NVME_SSD", "NETWORK", name="storagetype"),
            nullable=False,
            comment="High-level category of the storage, e.g. HDD or SDD.",
        ),
        sa.Column(
            "max_iops",
            sa.Integer(),
            nullable=True,
            comment="Maximum Input/Output Operations Per Second.",
        ),
        sa.Column(
            "max_throughput",
            sa.Integer(),
            nullable=True,
            comment="Maximum Throughput (MiB/s).",
        ),
        sa.Column(
            "min_size",
            sa.Integer(),
            nullable=True,
            comment="Minimum required size (GiB).",
        ),
        sa.Column(
            "max_size",
            sa.Integer(),
            nullable=True,
            comment="Maximum possible size (GiB).",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_storage_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "storage_id", "observed_at", name=op.f("pk_storage_scd")
        ),
        comment="SCD version of .tables.StorageScd.",
    )
    op.create_table(
        "traffic_price_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "direction",
            sa.Enum("IN", "OUT", name="trafficdirection"),
            nullable=False,
            comment="Direction of the traffic: inbound or outbound.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_traffic_price_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "datacenter_id",
            "direction",
            "observed_at",
            name=op.f("pk_traffic_price_scd"),
        ),
        comment="SCD version of .tables.TrafficPriceScd.",
    )
    op.create_table(
        "vendor_compliance_link",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "compliance_framework_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Compliance Framework.",
        ),
        sa.Column(
            "comment",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional references, such as dates, URLs, and additional information/evidence.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["compliance_framework_id"],
            ["compliance_framework.compliance_framework_id"],
            name=op.f(
                "fk_vendor_compliance_link_compliance_framework_id_compliance_framework"
            ),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_vendor_compliance_link_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "compliance_framework_id",
            name=op.f("pk_vendor_compliance_link"),
        ),
        comment="List of known Compliance Frameworks paired with vendors.",
    )
    op.create_table(
        "vendor_compliance_link_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "compliance_framework_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Compliance Framework.",
        ),
        sa.Column(
            "comment",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional references, such as dates, URLs, and additional information/evidence.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["compliance_framework_id"],
            ["compliance_framework.compliance_framework_id"],
            name=op.f(
                "fk_vendor_compliance_link_scd_compliance_framework_id_compliance_framework"
            ),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_vendor_compliance_link_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "compliance_framework_id",
            "observed_at",
            name=op.f("pk_vendor_compliance_link_scd"),
        ),
        comment="SCD version of .tables.VendorComplianceLinkScd.",
    )
    op.create_table(
        "zone_scd",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "zone_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "api_reference",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="How this resource is referenced in the vendor API calls. This is usually either the id or name of the resource, depening on the vendor and actual API endpoint.",
        ),
        sa.Column(
            "display_name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly reference (usually the id or name) of the resource.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_zone_scd_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "datacenter_id",
            "zone_id",
            "observed_at",
            name=op.f("pk_zone_scd"),
        ),
        comment="SCD version of .tables.ZoneScd.",
    )
    op.create_table(
        "benchmark_score",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "server_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Server.",
        ),
        sa.Column(
            "benchmark_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Benchmark.",
        ),
        sa.Column(
            "config",
            sa.JSON(),
            nullable=False,
            comment='Dictionary of config parameters of the specific benchmark, e.g. {"bandwidth": 4096}',
        ),
        sa.Column(
            "score",
            sa.Float(),
            nullable=False,
            comment="The resulting score of the benchmark.",
        ),
        sa.Column(
            "note",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=True,
            comment="Optional note, comment or context on the benchmark score.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["benchmark_id"],
            ["benchmark.benchmark_id"],
            name=op.f("fk_benchmark_score_benchmark_id_benchmark"),
        ),
        sa.ForeignKeyConstraint(
            ["server_id"],
            ["server.server_id"],
            name=op.f("fk_benchmark_score_server_id_server"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "server_id"],
            ["benchmark_score.vendor_id", "benchmark_score.server_id"],
            name=op.f("fk_benchmark_score_vendor_id_benchmark_score"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_benchmark_score_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "server_id",
            "benchmark_id",
            "config",
            name=op.f("pk_benchmark_score"),
        ),
        comment="Results of running Benchmark scenarios on Servers.",
    )
    op.create_table(
        "ipv4_price",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "datacenter_id"],
            ["datacenter.vendor_id", "datacenter.datacenter_id"],
            name=op.f("fk_ipv4_price_vendor_id_datacenter"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_ipv4_price_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", name=op.f("pk_ipv4_price")
        ),
        comment="Price of an IPv4 address in each Datacenter.",
    )
    op.create_table(
        "storage_price",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "storage_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Storage.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "datacenter_id"],
            ["datacenter.vendor_id", "datacenter.datacenter_id"],
            name=op.f("fk_storage_price_vendor_id_datacenter"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "storage_id"],
            ["storage.vendor_id", "storage.storage_id"],
            name=op.f("fk_storage_price_vendor_id_storage"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_storage_price_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", "storage_id", name=op.f("pk_storage_price")
        ),
        comment="Flexible Storage prices in each Datacenter.",
    )
    op.create_table(
        "traffic_price",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "direction",
            sa.Enum("IN", "OUT", name="trafficdirection"),
            nullable=False,
            comment="Direction of the traffic: inbound or outbound.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "datacenter_id"],
            ["datacenter.vendor_id", "datacenter.datacenter_id"],
            name=op.f("fk_traffic_price_vendor_id_datacenter"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_traffic_price_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", "direction", name=op.f("pk_traffic_price")
        ),
        comment="Extra Traffic prices in each Datacenter.",
    )
    op.create_table(
        "zone",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "zone_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Unique identifier, as called at the Vendor.",
        ),
        sa.Column(
            "name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly name.",
        ),
        sa.Column(
            "api_reference",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="How this resource is referenced in the vendor API calls. This is usually either the id or name of the resource, depening on the vendor and actual API endpoint.",
        ),
        sa.Column(
            "display_name",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Human-friendly reference (usually the id or name) of the resource.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "datacenter_id"],
            ["datacenter.vendor_id", "datacenter.datacenter_id"],
            name=op.f("fk_zone_vendor_id_datacenter"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"], ["vendor.vendor_id"], name=op.f("fk_zone_vendor_id_vendor")
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id", "datacenter_id", "zone_id", name=op.f("pk_zone")
        ),
        comment="Availability zones of Datacenters.",
    )
    op.create_table(
        "server_price",
        sa.Column(
            "vendor_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Vendor.",
        ),
        sa.Column(
            "datacenter_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Datacenter.",
        ),
        sa.Column(
            "zone_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Zone.",
        ),
        sa.Column(
            "server_id",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Reference to the Server.",
        ),
        sa.Column(
            "operating_system",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Operating System.",
        ),
        sa.Column(
            "allocation",
            sa.Enum("ONDEMAND", "RESERVED", "SPOT", name="allocation"),
            nullable=False,
            comment="Allocation method, e.g. on-demand or spot.",
        ),
        sa.Column(
            "unit",
            sa.Enum("YEAR", "MONTH", "HOUR", "GIB", "GB", "GB_MONTH", name="priceunit"),
            nullable=False,
            comment="Billing unit of the pricing model.",
        ),
        sa.Column(
            "price",
            sa.Float(),
            nullable=False,
            comment="Actual price of a billing unit.",
        ),
        sa.Column(
            "price_upfront",
            sa.Float(),
            nullable=False,
            comment="Price to be paid when setting up the resource.",
        ),
        sa.Column(
            "price_tiered",
            sa.JSON(),
            nullable=False,
            comment="List of pricing tiers with min/max thresholds and actual prices.",
        ),
        sa.Column(
            "currency",
            sqlmodel.sql.sqltypes.AutoString(),
            nullable=False,
            comment="Currency of the prices.",
        ),
        sa.Column(
            "status",
            sa.Enum("ACTIVE", "INACTIVE", name="status"),
            nullable=False,
            comment="Status of the resource (active or inactive).",
        ),
        sa.Column(
            "observed_at",
            sa.DateTime(),
            nullable=False,
            comment="Timestamp of the last observation.",
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "datacenter_id", "zone_id"],
            ["zone.vendor_id", "zone.datacenter_id", "zone.zone_id"],
            name=op.f("fk_server_price_vendor_id_zone"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "datacenter_id"],
            ["datacenter.vendor_id", "datacenter.datacenter_id"],
            name=op.f("fk_server_price_vendor_id_datacenter"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id", "server_id"],
            ["server.vendor_id", "server.server_id"],
            name=op.f("fk_server_price_vendor_id_server"),
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendor.vendor_id"],
            name=op.f("fk_server_price_vendor_id_vendor"),
        ),
        sa.PrimaryKeyConstraint(
            "vendor_id",
            "datacenter_id",
            "zone_id",
            "server_id",
            "allocation",
            name=op.f("pk_server_price"),
        ),
        comment="Server type prices per Datacenter and Allocation method.",
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # upgrade onlny named already existing constraints, no revert needed/can be done
    pass
